@@ -361,12 +361,13 @@ KTextEditor::View *KateViewSpace::createView(KTextEditor::Document *doc)
 
     // restore the config of this view if possible
     if (!m_group.isEmpty()) {
-        QString fn = v->document()->url().toString();
-        if (!fn.isEmpty()) {
-            QString vgroup = QStringLiteral("%1 %2").arg(m_group, fn);
-
-            KateSession::Ptr as = KateApp::self()->sessionManager()->activeSession();
-            if (as->config() && as->config()->hasGroup(vgroup)) {
+        if (KateSession::Ptr as = KateApp::self()->sessionManager()->activeSession(); as->config()) {
+            // try id, fallback to url for old configs
+            QString vgroup = QStringLiteral("%1 %2").arg(m_group, KateApp::self()->documentManager()->documentInfo(v->document())->sessionConfigId);
+            if (!as->config()->hasGroup(vgroup)) {
+                vgroup = QStringLiteral("%1 %2").arg(m_group, v->document()->url().toString());
+            }
+            if (as->config()->hasGroup(vgroup)) {
                 KConfigGroup cg(as->config(), vgroup);
                 v->readSessionConfig(cg);
             }
@@ -1222,19 +1223,22 @@ void KateViewSpace::showContextMenu(int idx, const QPoint &globalPos)
 
 void KateViewSpace::saveConfig(KConfigBase *config, int myIndex, const QString &viewConfGrp)
 {
-    //   qCDebug(LOG_KATE)<<"KateViewSpace::saveConfig("<<myIndex<<", "<<viewConfGrp<<") - currentView: "<<currentView()<<")";
-    QString groupname = QString(viewConfGrp + QStringLiteral("-ViewSpace %1")).arg(myIndex);
+    const QString groupname = QString(viewConfGrp + QStringLiteral("-ViewSpace %1")).arg(myIndex);
 
-    // aggregate all views in view space (LRU ordered)
+    // aggregate all registered documents & views in view space (LRU ordered)
+    // we need even the documents without tabs to avoid that we later have issues with closing
+    // documents and not the right ones being used as replacement if you have a limit on tabs
     std::vector<KTextEditor::View *> views;
     QStringList lruList;
-    const auto docList = documentList();
-    for (DocOrWidget docOrWidget : docList) {
-        if (docOrWidget.widget()) {
+    const auto docList = m_tabBar->lruSortedDocuments();
+    for (auto doc : docList) {
+        // we can only store stuff about documents that get an id
+        const int sessionId = KateApp::self()->documentManager()->documentInfo(doc)->sessionConfigId;
+        if (sessionId < 0) {
             continue;
         }
-        auto doc = docOrWidget.doc();
-        lruList << doc->url().toString();
+
+        lruList << QString::number(sessionId);
         auto it = m_docToView.find(doc);
         if (it != m_docToView.end()) {
             views.push_back(it->second);
@@ -1246,18 +1250,18 @@ void KateViewSpace::saveConfig(KConfigBase *config, int myIndex, const QString &
     group.writeEntry("Count", static_cast<int>(views.size()));
 
     if (currentView()) {
-        group.writeEntry("Active View", currentView()->document()->url().toString());
+        group.writeEntry("Active View", QString::number(KateApp::self()->documentManager()->documentInfo(currentView()->document())->sessionConfigId));
     }
 
     // Save file list, including cursor position in this instance.
     int idx = 0;
     for (auto view : views) {
-        const auto url = view->document()->url();
-        if (!url.isEmpty()) {
-            group.writeEntry(QStringLiteral("View %1").arg(idx), url.toString());
+        const int sessionId = KateApp::self()->documentManager()->documentInfo(view->document())->sessionConfigId;
+        if (sessionId >= 0) {
+            group.writeEntry(QStringLiteral("View %1").arg(idx), sessionId);
 
-            // view config, group: "ViewSpace <n> url"
-            QString vgroup = QStringLiteral("%1 %2").arg(groupname, url.toString());
+            // view config, group: "ViewSpace <n> id"
+            QString vgroup = QStringLiteral("%1 %2").arg(groupname, sessionId);
             KConfigGroup viewGroup(config, vgroup);
             view->writeSessionConfig(viewGroup);
         }
@@ -1281,25 +1285,39 @@ void KateViewSpace::restoreConfig(KateViewManager *viewMan, const KConfigBase *c
 
     // restore Document lru list so that all tabs from the last session reappear
     const QStringList lruList = group.readEntry("Documents", QStringList());
-    for (const auto &url : lruList) {
-        // ignore untitled stuff
-        if (url.isEmpty()) {
+    for (const auto &idOrUrl : lruList) {
+        // ignore stuff with no id or url
+        if (idOrUrl.isEmpty()) {
             continue;
         }
 
-        // ignore non-existing documents
-        if (auto doc = KateApp::self()->documentManager()->findDocument(QUrl(url))) {
+        // if id use that, is the new format to allow to differentiate between different untitled documents
+        bool isInt = false;
+        KTextEditor::Document *doc = nullptr;
+        if (const int id = idOrUrl.toInt(&isInt, 10); isInt) {
+            doc = KateApp::self()->documentManager()->findDocumentForSessionConfigId(id);
+        } else {
+            doc = KateApp::self()->documentManager()->findDocument(QUrl(idOrUrl));
+        }
+        if (doc) {
             registerDocument(doc);
         }
     }
 
     // restore active view properties
-    const QString fn = group.readEntry("Active View");
-    if (!fn.isEmpty()) {
-        if (auto doc = KateApp::self()->documentManager()->findDocument(QUrl(fn))) {
+    if (const QString idOrUrl = group.readEntry("Active View"); !idOrUrl.isEmpty()) {
+        // if id use that, is the new format to allow to differentiate between different untitled documents
+        bool isInt = false;
+        KTextEditor::Document *doc = nullptr;
+        if (const int id = idOrUrl.toInt(&isInt, 10); isInt) {
+            doc = KateApp::self()->documentManager()->findDocumentForSessionConfigId(id);
+        } else {
+            doc = KateApp::self()->documentManager()->findDocument(QUrl(idOrUrl));
+        }
+        if (doc) {
             if (auto view = viewMan->createView(doc, this)) {
                 // view config, group: "ViewSpace <n> url"
-                const QString vgroup = QStringLiteral("%1 %2").arg(groupname, fn);
+                const QString vgroup = QStringLiteral("%1 %2").arg(groupname, idOrUrl);
                 view->readSessionConfig(KConfigGroup(config, vgroup));
                 m_tabBar->setCurrentDocument(doc);
             }
